@@ -5,6 +5,7 @@ import os
 import numpy as np
 import pandas as pd
 from typing import Dict, List
+from processing.teams.teams import enrich_with_geographic_and_temporal_features
 
 class MatchStatsCalculator:
     """
@@ -14,21 +15,15 @@ class MatchStatsCalculator:
 
     def __init__(
         self,
-        # matches_csv_path: str,
-        # players_csv_path: str,
-        # keepers_csv_path: str,
-        # players_features_csv_path: str,
-        # keepers_features_csv_path: str,
-        # ranking_csv_path: str,
-        # trophies_csv_path: str,
+        matches_csv_path: str,
+        stadiums_csv_path: str,
         dataset_global_csv_path: str,
-        dataset_country_csv_path: str,
-        dataset_competition_csv_path: str,
         comp_type: str,
         comp_country: str,
         features_config: Dict[str, any],
-        teams: str
     ):
+        self.matches_csv_path = matches_csv_path
+        self.stadiums_csv_path = stadiums_csv_path
         self.columns_to_copy = features_config["fbref_informations"]
         self.comp_type = comp_type
         self.comp_country = comp_country
@@ -37,9 +32,6 @@ class MatchStatsCalculator:
             for metric, config in features_config["fbref_features"].items()
         }
         self.existing_csv = dataset_global_csv_path
-        self.existing_csv_country = dataset_country_csv_path
-        self.existing_csv_competition = dataset_competition_csv_path
-        self.teams = teams
 
     # ----------------------------- Core stats -----------------------------
     def pythagorean_expectation(self, goals_for: float, goals_against: float, exponent: float = 1.7) -> float:
@@ -756,13 +748,9 @@ class MatchStatsCalculator:
     # ------------------------------ Updates -------------------------------
     def update_columns(
         self, 
-        combined_df: pd.DataFrame, 
-        combined_df_country: pd.DataFrame, 
-        combined_df_competition: pd.DataFrame, 
+        combined_df: pd.DataFrame,
         choice_2=None, 
-        start_index_global=0,
-        start_index_country=0,
-        start_index_competition=0
+        start_index_global=0
     ):
         """
         Calculate features for specified rows in the dataset.
@@ -770,17 +758,11 @@ class MatchStatsCalculator:
         
         Args:
             combined_df: Global dataset
-            combined_df_country: Country-specific dataset
-            combined_df_competition: Competition-specific dataset
             choice_2: Number of new rows to process
             start_index_global: Starting index for global dataset
-            start_index_country: Starting index for country dataset
-            start_index_competition: Starting index for competition dataset
         """
 
         df = combined_df.copy()
-        df_country = combined_df_country.copy()
-        df_competition = combined_df_competition.copy()
 
         if df.empty:
             print("[INFO] No data in DataFrame; skipping feature building.")
@@ -790,10 +772,6 @@ class MatchStatsCalculator:
         # Normalize dates
         if "date_of_match" in df.columns:
             df["date_of_match"] = pd.to_datetime(df["date_of_match"], errors="coerce")
-        if "date_of_match" in df_country.columns:
-            df_country["date_of_match"] = pd.to_datetime(df_country["date_of_match"], errors="coerce")
-        if "date_of_match" in df_competition.columns:
-            df_competition["date_of_match"] = pd.to_datetime(df_competition["date_of_match"], errors="coerce")
 
         # Check if competition column exists
         if "competition" not in df.columns:
@@ -808,241 +786,34 @@ class MatchStatsCalculator:
             end_index_global = start_index_global + choice_2
             indices_global = list(range(start_index_global, end_index_global))
             
-            # Índices para COUNTRY dataset
-            end_index_country = start_index_country + choice_2
-            indices_country = list(range(start_index_country, end_index_country))
-            
-            # Índices para COMPETITION dataset
-            end_index_competition = start_index_competition + choice_2
-            indices_competition = list(range(start_index_competition, end_index_competition))
-            
-            df_to_process = df.iloc[start_index_global:end_index_global].copy()
         else:
             indices_global = list(range(len(df)))
-            indices_country = list(range(len(df_country)))
-            indices_competition = list(range(len(df_competition)))
-            df_to_process = df.copy()
 
         # =========================================================================
-        # TEAM PRE-PROCESSING
+        # ENRIQUECIMIENTO CON FEATURES GEOGRÁFICAS Y TEMPORALES
         # =========================================================================
-        unique_home_teams = df_to_process['home_team_name'].dropna().unique()
+        print("\n[FEATURES] Enriching with geographic and temporal features...")
 
-        if os.path.exists(self.teams):
-            df_teams = pd.read_csv(self.teams, encoding='utf-8')
-        else:
-            df_teams = pd.DataFrame(columns=['name', 'country', 'longitude', 'latitude', 'altitude'])
-            os.makedirs(os.path.dirname(self.teams), exist_ok=True)
+        df = enrich_with_geographic_and_temporal_features(
+            df_matches=df,
+            df_all_matches=combined_df,
+            stadiums_path=self.stadiums_csv_path,
+            historical_matches_path=self.matches_csv_path,
+            verbose=True
+        )
 
-        for team_name in unique_home_teams:
-            if pd.isna(team_name) or team_name == '':
-                continue
-            
-            team_info = df_teams[df_teams['name'] == team_name]
-            
-            if team_info.empty:
-                # ===== CASE 1: NEW TEAM =====
-                print(f"                New team found: {team_name}")
-                
-                # Search for team country in previous league matches
-                team_country = None
-                
-                # Search in existing_matches where the team played (home or away) in a league
-                if not combined_df.empty and 'competition_type' in combined_df.columns:
-                    league_matches = combined_df[
-                        (combined_df['competition_type'] == 'league') &
-                        ((combined_df['home_team_name'] == team_name) | 
-                        (combined_df['away_team_name'] == team_name))
-                    ]
-                                        
-                    if not league_matches.empty:
-                        # Sort by date (most recent first) if date column exists
-                        if 'date_of_match' in league_matches.columns:
-                            league_matches = league_matches.sort_values('date_of_match', ascending=False)
-                        
-                        # Take country from most recent match
-                        if 'competition_country' in league_matches.columns:
-                            team_country = league_matches.iloc[0]['competition_country']
-                        else:
-                            print(f"                  competition_country column not found in existing matches")
-                    else:
-                        print(f"                  No previous league matches found for {team_name}")
-                else:
-                    print(f"                  No existing matches available for lookup")
-                
-                # If country not found, ask user
-                if not team_country or pd.isna(team_country):
-                    print(f"                Please provide the country for this team:")
-                    
-                    while True:
-                        country_input = input(f"                  Country (e.g., England, Spain, Germany): ").strip()
-                        if country_input != '':
-                            team_country = country_input
-                            break
-                        else:
-                            print("                  Country cannot be empty. Please enter a valid country.")
-                
-                # Now ask for stadium coordinates
-                print(f"                Please provide home stadium location:")
-                
-                while True:
-                    try:
-                        longitude_input = input(f"                  Longitude (decimal degrees, e.g., -3.688): ").strip()
-                        if longitude_input == '':
-                            longitude = None
-                            break
-                        longitude = float(longitude_input)
-                        break
-                    except ValueError:
-                        print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                
-                while True:
-                    try:
-                        latitude_input = input(f"                  Latitude (decimal degrees, e.g., 40.453): ").strip()
-                        if latitude_input == '':
-                            latitude = None
-                            break
-                        latitude = float(latitude_input)
-                        break
-                    except ValueError:
-                        print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                
-                while True:
-                    try:
-                        altitude_input = input(f"                  Altitude (meters, e.g., 667): ").strip()
-                        if altitude_input == '':
-                            altitude = None
-                            break
-                        altitude = float(altitude_input)
-                        break
-                    except ValueError:
-                        print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                
-                # Create new team with country included
-                new_team = {
-                    'name': team_name,
-                    'country': team_country,
-                    'longitude': longitude,
-                    'latitude': latitude,
-                    'altitude': altitude
-                }
-
-                # Append using loc with next index
-                df_teams.loc[len(df_teams)] = new_team
-                df_teams.to_csv(self.teams, index=False, encoding='utf-8')
-            
-            else:
-                # ===== CASE 2: TEAM EXISTS BUT MAY HAVE INCOMPLETE DATA =====
-                team_idx = team_info.index[0]
-                current_country = team_info.iloc[0]['country']
-                current_longitude = team_info.iloc[0]['longitude']
-                current_latitude = team_info.iloc[0]['latitude']
-                current_altitude = team_info.iloc[0]['altitude']
-                
-                needs_update = False
-                
-                # Check if country is missing
-                if pd.isna(current_country) or current_country == '' or current_country is None:
-                    print(f"                Team '{team_name}' missing country information")
-                    needs_update = True
-                    
-                    # Search for team country in previous league matches
-                    team_country = None
-                    
-                    if not combined_df.empty and 'competition_type' in combined_df.columns:
-                        league_matches = combined_df[
-                            (combined_df['competition_type'] == 'league') &
-                            ((combined_df['home_team_name'] == team_name) | 
-                            (combined_df['away_team_name'] == team_name))
-                        ]
-                                            
-                        if not league_matches.empty:
-                            if 'date_of_match' in league_matches.columns:
-                                league_matches = league_matches.sort_values('date_of_match', ascending=False)
-                            
-                            if 'competition_country' in league_matches.columns:
-                                team_country = league_matches.iloc[0]['competition_country']
-                    
-                    # If country not found, ask user
-                    if not team_country or pd.isna(team_country):
-                        print(f"                Please provide the country for this team:")
-                        
-                        while True:
-                            country_input = input(f"                  Country (e.g., England, Spain, Germany): ").strip()
-                            if country_input != '':
-                                team_country = country_input
-                                break
-                            else:
-                                print("                  Country cannot be empty. Please enter a valid country.")
-                    
-                    df_teams.at[team_idx, 'country'] = team_country
-                
-                # Check if coordinates are missing
-                if pd.isna(current_longitude) or pd.isna(current_latitude) or pd.isna(current_altitude):
-                    print(f"                Team '{team_name}' missing location information")
-                    needs_update = True
-                    print(f"                Please provide home stadium location:")
-                    
-                    # Ask for longitude if missing
-                    if pd.isna(current_longitude):
-                        while True:
-                            try:
-                                longitude_input = input(f"                  Longitude (decimal degrees, e.g., -3.688): ").strip()
-                                if longitude_input == '':
-                                    df_teams.at[team_idx, 'longitude'] = None
-                                    break
-                                df_teams.at[team_idx, 'longitude'] = float(longitude_input)
-                                break
-                            except ValueError:
-                                print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                    
-                    # Ask for latitude if missing
-                    if pd.isna(current_latitude):
-                        while True:
-                            try:
-                                latitude_input = input(f"                  Latitude (decimal degrees, e.g., 40.453): ").strip()
-                                if latitude_input == '':
-                                    df_teams.at[team_idx, 'latitude'] = None
-                                    break
-                                df_teams.at[team_idx, 'latitude'] = float(latitude_input)
-                                break
-                            except ValueError:
-                                print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                    
-                    # Ask for altitude if missing
-                    if pd.isna(current_altitude):
-                        while True:
-                            try:
-                                altitude_input = input(f"                  Altitude (meters, e.g., 667): ").strip()
-                                if altitude_input == '':
-                                    df_teams.at[team_idx, 'altitude'] = None
-                                    break
-                                df_teams.at[team_idx, 'altitude'] = float(altitude_input)
-                                break
-                            except ValueError:
-                                print("                  Invalid input. Please enter a valid number or press Enter to skip.")
-                
-                # Save if any updates were made
-                if needs_update:
-                    df_teams.to_csv(self.teams, index=False, encoding='utf-8')
-                    print(f"                Team '{team_name}' information updated")
+        print("[FEATURES] Geographic and temporal features added successfully\n")
 
         # =========================================================================
         # PROCESAMIENTO DE FEATURES
         # =========================================================================
         # Storage separado para cada dataset
         new_values_global = {}
-        new_values_country = {}
-        new_values_competition = {}
 
         home_away_location = ["home", "away"]
 
         # Iterate using GLOBAL dataset indices
         for i, df_idx_global in enumerate(indices_global):
-            
-            # Corresponding indices in other datasets
-            df_idx_country = indices_country[i]
-            df_idx_competition = indices_competition[i]
 
             # Work with the row from GLOBAL dataset
             row_df = df.iloc[[df_idx_global]].copy()
@@ -1103,8 +874,6 @@ class MatchStatsCalculator:
                             new_col_name = f"{col_name}_global" if not col_name.endswith("_global") else col_name
                             # Guardar en los 3 datasets con SUS índices correspondientes
                             new_values_global.setdefault(new_col_name, {})[df_idx_global] = val
-                            new_values_country.setdefault(new_col_name, {})[df_idx_country] = val
-                            new_values_competition.setdefault(new_col_name, {})[df_idx_competition] = val
 
                     except Exception as e:
                         print(f"  [ERROR] Row {df_idx_global}, {home_away} {base_metric} (global): {e}")
@@ -1141,8 +910,6 @@ class MatchStatsCalculator:
                             for col_name, val in features_comp.items():
                                 new_col_name = f"{col_name}_comp" if not col_name.endswith("_comp") else col_name
                                 new_values_global.setdefault(new_col_name, {})[df_idx_global] = val
-                                new_values_country.setdefault(new_col_name, {})[df_idx_country] = val
-                                new_values_competition.setdefault(new_col_name, {})[df_idx_competition] = val
 
                         except Exception as e:
                             print(f"  [ERROR] Row {df_idx_global}, {home_away} {base_metric} (comp): {e}")
@@ -1161,8 +928,6 @@ class MatchStatsCalculator:
                     for col_name, val in pytha_global.items():
                         new_col_name = f"{col_name}_global"
                         new_values_global.setdefault(new_col_name, {})[df_idx_global] = val
-                        new_values_country.setdefault(new_col_name, {})[df_idx_country] = val
-                        new_values_competition.setdefault(new_col_name, {})[df_idx_competition] = val
                 except Exception as e:
                     print(f"  [ERROR] Row {df_idx_global}, {home_away} Pythagorean (global): {e}")
 
@@ -1182,8 +947,6 @@ class MatchStatsCalculator:
                         for col_name, val in pytha_comp.items():
                             new_col_name = f"{col_name}_comp"
                             new_values_global.setdefault(new_col_name, {})[df_idx_global] = val
-                            new_values_country.setdefault(new_col_name, {})[df_idx_competition] = val
-                            new_values_competition.setdefault(new_col_name, {})[df_idx_competition] = val
                     except Exception as e:
                         print(f"  [ERROR] Row {df_idx_global}, {home_away} Pythagorean (comp): {e}")
 
@@ -1193,27 +956,15 @@ class MatchStatsCalculator:
         
         # Convertir a DataFrames
         new_values_df_global = pd.DataFrame.from_dict(new_values_global, orient="columns")
-        new_values_df_country = pd.DataFrame.from_dict(new_values_country, orient="columns")
-        new_values_df_competition = pd.DataFrame.from_dict(new_values_competition, orient="columns")
 
         # Identificar columnas faltantes
         missing_cols_global = [col for col in new_values_df_global.columns if col not in df.columns]
-        missing_cols_country = [col for col in new_values_df_country.columns if col not in df_country.columns]
-        missing_cols_competition = [col for col in new_values_df_competition.columns if col not in df_competition.columns]
 
         # Agregar todas las columnas faltantes de una vez usando concat
         if missing_cols_global:
             missing_df = pd.DataFrame(np.nan, index=df.index, columns=missing_cols_global)
             df = pd.concat([df, missing_df], axis=1)
 
-        if missing_cols_country:
-            missing_df = pd.DataFrame(np.nan, index=df_country.index, columns=missing_cols_country)
-            df_country = pd.concat([df_country, missing_df], axis=1)
-
-        if missing_cols_competition:
-            missing_df = pd.DataFrame(np.nan, index=df_competition.index, columns=missing_cols_competition)
-            df_competition = pd.concat([df_competition, missing_df], axis=1)
-
         # =========================================================================
         # Actualizar valores en cada DataFrame
         # =========================================================================
@@ -1223,50 +974,31 @@ class MatchStatsCalculator:
             for idx in new_values_df_global.index:
                 if idx in df.index:
                     df.at[idx, col] = new_values_df_global.at[idx, col]
-
-        # Actualizar valores en COUNTRY
-        for col in new_values_df_country.columns:
-            for idx in new_values_df_country.index:
-                if idx in df_country.index:
-                    df_country.at[idx, col] = new_values_df_country.at[idx, col]
-
-        # Actualizar valores en COMPETITION
-        for col in new_values_df_competition.columns:
-            for idx in new_values_df_competition.index:
-                if idx in df_competition.index:
-                    df_competition.at[idx, col] = new_values_df_competition.at[idx, col]
         
         basic_cols = [col for col in self.columns_to_copy if col in df.columns]
         feature_cols = [col for col in df.columns if col not in basic_cols]
         ordered_cols = basic_cols + feature_cols
         
         df = df[ordered_cols]
-        df_country = df_country[ordered_cols]
-        df_competition = df_competition[ordered_cols]
 
         # Guardar los 3 DataFrames
         df.to_csv(self.existing_csv, index=False)
-        
-        df_country.to_csv(self.existing_csv_country, index=False)
-        
-        df_competition.to_csv(self.existing_csv_competition, index=False)
 
     def data_update_dataset(
-        self, match_df: pd.DataFrame, injured: List
+        self, match_df: pd.DataFrame
     ) -> pd.DataFrame:
         """
         Update dataset with new matches and calculate features.
 
         Args:
             match_df: DataFrame with newly scraped matches
-            injured: List of injured players
 
         Returns:
             DataFrame with new matches and calculated features
         """
 
         # =========================================================================
-        # 1. Load existing datasets
+        # 1. Load existing dataset
         # =========================================================================
         
         # GLOBAL dataset
@@ -1277,22 +1009,6 @@ class MatchStatsCalculator:
             existing_df = pd.DataFrame()
             original_length = 0
 
-        # COUNTRY dataset
-        if os.path.exists(self.existing_csv_country):
-            existing_country_df = pd.read_csv(self.existing_csv_country, low_memory=False)
-            original_length_country = len(existing_country_df)
-        else:
-            existing_country_df = pd.DataFrame()
-            original_length_country = 0
-
-        # COMPETITION dataset
-        if os.path.exists(self.existing_csv_competition):
-            existing_competition_df = pd.read_csv(self.existing_csv_competition, low_memory=False)
-            original_length_competition = len(existing_competition_df)
-        else:
-            existing_competition_df = pd.DataFrame()
-            original_length_competition = 0
-
         # =========================================================================
         # 2. Extract only needed columns from match_df
         # =========================================================================
@@ -1326,8 +1042,6 @@ class MatchStatsCalculator:
         # Obtener todas las columnas únicas
         all_cols = list(set(
             existing_df.columns.tolist() + 
-            existing_country_df.columns.tolist() + 
-            existing_competition_df.columns.tolist() + 
             new_rows.columns.tolist()
         ))
         
@@ -1337,16 +1051,6 @@ class MatchStatsCalculator:
             missing_df = pd.DataFrame(np.nan, index=existing_df.index, columns=missing_cols_existing)
             existing_df = pd.concat([existing_df, missing_df], axis=1)
 
-        missing_cols_country = [col for col in all_cols if col not in existing_country_df.columns]
-        if missing_cols_country:
-            missing_df = pd.DataFrame(np.nan, index=existing_country_df.index, columns=missing_cols_country)
-            existing_country_df = pd.concat([existing_country_df, missing_df], axis=1)
-
-        missing_cols_competition = [col for col in all_cols if col not in existing_competition_df.columns]
-        if missing_cols_competition:
-            missing_df = pd.DataFrame(np.nan, index=existing_competition_df.index, columns=missing_cols_competition)
-            existing_competition_df = pd.concat([existing_competition_df, missing_df], axis=1)
-
         missing_cols_new = [col for col in all_cols if col not in new_rows.columns]
         if missing_cols_new:
             missing_df = pd.DataFrame(np.nan, index=new_rows.index, columns=missing_cols_new)
@@ -1360,16 +1064,6 @@ class MatchStatsCalculator:
         else:
             combined_df_global = pd.concat([existing_df[all_cols], new_rows[all_cols]], ignore_index=True)
 
-        if existing_country_df.empty:
-            combined_df_country = new_rows[all_cols].copy()
-        else:
-            combined_df_country = pd.concat([existing_country_df[all_cols], new_rows[all_cols]], ignore_index=True)
-
-        if existing_competition_df.empty:
-            combined_df_competition = new_rows[all_cols].copy()
-        else:
-            combined_df_competition = pd.concat([existing_competition_df[all_cols], new_rows[all_cols]], ignore_index=True)
-
         # Columnas básicas (de columns_to_copy) que deben ir primero
         basic_cols = [col for col in self.columns_to_copy if col in combined_df_global.columns]
         
@@ -1381,26 +1075,18 @@ class MatchStatsCalculator:
         
         # Reordenar los 3 DataFrames
         combined_df_global = combined_df_global[ordered_cols]
-        combined_df_country = combined_df_country[ordered_cols]
-        combined_df_competition = combined_df_competition[ordered_cols]
 
         # =========================================================================
-        # 8. Calcular features SOLO para nuevas filas en CADA dataset
+        # 8. Calcular features SOLO para nuevas filas en el dataset
         # =========================================================================
         start_index_global = original_length
-        start_index_country = original_length_country
-        start_index_competition = original_length_competition
         choice_2 = len(new_rows)
 
-        # Llamar update_columns con los índices correctos para cada dataset
+        # Llamar update_columns con los índices correctos para el dataset
         self.update_columns(
-            combined_df_global, 
-            combined_df_country, 
-            combined_df_competition, 
+            combined_df_global,
             choice_2=choice_2, 
-            start_index_global=start_index_global,
-            start_index_country=start_index_country,
-            start_index_competition=start_index_competition
+            start_index_global=start_index_global
         )
 
         return new_rows
\ No newline at end of file
