@@ -9,12 +9,14 @@ from typing import Optional, List, Dict, Tuple, Any
 import pandas as pd
 
 # Import custom modules
-from scrapers.fbref_scraper import FBRefScraper
+from scrapers.sofascore_scraper import SofaScoreScraper
 from utils.date import get_previous_days, validate_execution_day, season_from_date, sort_by_match_datetime
-from processing.teams.teams import enrich_with_team_history, update_league_winners
+from processing.teams.teams import enrich_with_team_history, update_league_winners, update_european_winners
 from utils.format import _format_paths
 from processing.ranking.league_ranking import LeagueRanking
 
+import traceback
+
 class DatasetUpdater:
     """Main class to orchestrate the dataset update process."""
 
@@ -104,10 +106,18 @@ class DatasetUpdater:
 
         matches = pd.DataFrame()
 
+        # Read existing matches, combine with new ones, and save
+        all_matches = (
+            pd.read_csv(matches_csv_path)
+            if os.path.exists(matches_csv_path)
+            else pd.DataFrame()
+        )
+
         # Step 2: Process each competition
         for comp_info in competitions_list:
             comp_name = comp_info["name"]
             comp_country = comp_info["country"]
+            comp_gameweeks = comp_info["number_of_gameweeks"]
 
             # Only format the ranking path (it has placeholders)
             formatted_paths = _format_paths(
@@ -128,8 +138,10 @@ class DatasetUpdater:
 
             try:
                 comp_matches = self._process_competition(
+                    all_matches,
                     comp_name,
                     comp_country,
+                    comp_gameweeks,
                     matches_csv_path,   # Fixed path, same for all competitions
                     players_csv_path,   # Fixed path, same for all competitions
                     keepers_csv_path,   # Fixed path, same for all competitions
@@ -139,27 +151,27 @@ class DatasetUpdater:
                     last_season,
                 )
 
-                if len(comp_matches) > 0:
-                    comp_matches = sort_by_match_datetime(comp_matches)
+                if comp_matches is not None and len(comp_matches) > 0:
                     matches = pd.concat([matches, comp_matches], ignore_index=True)
+                    print(f"                Added {len(comp_matches)} matches from {comp_name}")
 
             except Exception as e:
+                print(f"                ERROR processing {comp_name}: {type(e).__name__}: {str(e)}")
+                traceback.print_exc()
                 continue
 
         # Check if any matches were found across ALL competitions
         if matches.empty:
             print("            No new matches found")
             return pd.DataFrame()
-
-        # Read existing matches, combine with new ones, and save
-        all_matches = (
-            pd.read_csv(matches_csv_path)
-            if os.path.exists(matches_csv_path)
-            else pd.DataFrame()
-        )
         
-        final_all_matches = pd.concat([matches, all_matches], ignore_index=True)
-        final_all_matches = sort_by_match_datetime(final_all_matches)
+        matches = sort_by_match_datetime(matches)
+        if all_matches.empty:
+            final_all_matches = matches.copy()
+        elif matches.empty:
+            final_all_matches = all_matches.copy()
+        else:
+            final_all_matches = pd.concat([all_matches, matches], ignore_index=True)
         
         final_all_matches.to_csv(matches_csv_path, index=False, encoding="utf-8")
         
@@ -183,6 +195,7 @@ class DatasetUpdater:
                 {
                     'name': 'liga',
                     'country': 'spain',
+                    'number_of_gameweeks': 38,
                     'paths': {
                         'ranking': 'https://...',
                         'teams_players': 'https://...',
@@ -220,7 +233,10 @@ class DatasetUpdater:
 
             # Extract country
             country = competition_config.get("country", "unknown")
-
+            
+            # Extract number of gameweeks
+            number_of_gameweeks = competition_config.get("number_of_gameweeks", 38)
+            
             # Extract and format information scraping URLs
             info_urls = competition_config.get("information_scraping_urls", {})
             formatted_info_urls = {}
@@ -238,6 +254,7 @@ class DatasetUpdater:
             comp_dict = {
                 "name": comp_name,
                 "country": country,
+                "number_of_gameweeks": number_of_gameweeks,
                 "information_urls": formatted_info_urls,
             }
 
@@ -247,8 +264,10 @@ class DatasetUpdater:
 
     def _process_competition(
         self,
+        all_matches: pd.DataFrame,
         comp_name: str,
         comp_country: str,
+        comp_gameweeks: Optional[int],
         matches_csv_path: str,
         players_csv_path: str,
         keepers_csv_path: str,
@@ -256,7 +275,7 @@ class DatasetUpdater:
         teams_csv_path: str,
         days: List[str],
         last_season: Tuple[str, int, int],
-    ) -> List[Dict]:
+    ) -> pd.DataFrame:
         """
         Process a single competition and return match data.
 
@@ -286,35 +305,35 @@ class DatasetUpdater:
         comp_type = self._infer_competition_type(comp_name)
 
         # Scrape match data from FBRef
-        scraper = FBRefScraper(
-            last_season, 
-            comp_config, 
-            comp_name, 
-            comp_config, 
-            comp_type, 
-            days
-        )
+        scraper = SofaScoreScraper(headless=True, verbose=False)
 
         # Run the scraper to get DataFrame-like objects
         df_matches, players_data, keepers_data = scraper.run_after()
 
+        df = scraper.scrape_matches(
+            url="https://www.sofascore.com/football/tournament/spain/laliga/8",
+            season="24/25",
+            max_rounds=38,
+            target_dates=days
+        )
+
         # If no matches found, stop processing
         if df_matches.empty:
             print(f"                No matches to process")
-            return
-
-        print(f"                Updating rankings...")
+            return pd.DataFrame()
+        
+        current_gameweek = int(df_matches['gameweek'].max())
 
         if comp_type == "league":
-            current_gameweek = int(df_matches['gameweek'].max())
 
-            # Check if gameweek is 38 BEFORE calling the function
-            if current_gameweek == 38:
+            # Check if gameweek is comp_gameweeks BEFORE calling the function
+            if current_gameweek == comp_gameweeks:
                 df_matches = scraper.scrape_league_standings(
                     ranking_csv_path=ranking_csv_path,
                     recent_matches_df=df_matches,
+                    all_matches_df=all_matches,
                     gameweek=current_gameweek,
-                    verbose=True
+                    verbose=False
                 )
 
                 df_matches = enrich_with_team_history(
@@ -327,7 +346,8 @@ class DatasetUpdater:
                 update_league_winners(
                     ranking_csv_path=ranking_csv_path,
                     trophies_csv_path=teams_csv_path,
-                    verbose=True
+                    comp_gameweeks=comp_gameweeks,
+                    verbose=False
                 )
             else:
                 manager = LeagueRanking()
@@ -352,6 +372,13 @@ class DatasetUpdater:
                 teams_csv_path,
                 comp_country
             )
+            if current_gameweek == comp_gameweeks:
+                update_european_winners(
+                    trophies_csv_path=teams_csv_path,
+                    final_match_df=df_matches,
+                    competition_name=comp_name,
+                    verbose=False
+                )
 
         # --- Ensure parent folder exists for players CSVs ---
         _parent = os.path.dirname(players_csv_path)
@@ -388,24 +415,52 @@ class DatasetUpdater:
                 # Read existing data
                 df_existing = pd.read_csv(players_csv_path, encoding="utf-8")
                 
-                # Accumulate yellow cards
+                # Accumulate yellow cards PER COMPETITION_TYPE (league vs european)
                 if 'Players' in df_existing.columns and 'PlayersYellowCards' in df_existing.columns:
                     if 'Players' in df_players.columns and 'PlayersYellowCards' in df_players.columns:
-                        # Create dict with previous yellow cards per player
-                        previous_yellows = df_existing.groupby('Players')['PlayersYellowCards'].max().to_dict()
+                        # Create dict with previous yellow cards per player PER COMPETITION_TYPE
+                        # Key: (player_name, competition_type, team_name), Value: max yellow cards
+                        if 'competition_type' in df_existing.columns and 'team_name' in df_existing.columns:
+                            previous_yellows = df_existing.groupby(['Players', 'competition_type', 'team_name'])['PlayersYellowCards'].max().to_dict()
+                        else:
+                            # Fallback if columns don't exist
+                            previous_yellows = df_existing.groupby('Players')['PlayersYellowCards'].max().to_dict()
                         
                         # For each new player record, accumulate yellow cards
                         def accumulate_yellows(row):
                             player_name = row['Players']
                             current_yellows = row.get('PlayersYellowCards', 0)
+                            gameweek = row.get('gameweek', None)
                             
-                            # If player exists in previous data
-                            if player_name in previous_yellows:
-                                prev_yellows = previous_yellows[player_name]
+                            # CRITICAL: If gameweek is 1, start fresh (new season)
+                            if pd.notna(gameweek) and int(float(gameweek)) == 1:
+                                # New season starts, don't accumulate
+                                return current_yellows
+                            
+                            # Get competition_type and team_name if available
+                            if ('competition_type' in row.index and 'team_name' in row.index and 
+                                'competition_type' in df_existing.columns and 'team_name' in df_existing.columns):
+                                competition_type = row['competition_type']
+                                team_name = row['team_name']
+                                key = (player_name, competition_type, team_name)
+                            else:
+                                key = player_name
+                            
+                            # If player exists in previous data for this competition_type and team
+                            if key in previous_yellows:
+                                prev_yellows = previous_yellows[key]
                                 # If there's an increment, add it to the previous total
                                 if pd.notna(current_yellows) and pd.notna(prev_yellows):
-                                    # Accumulate: previous + new cards from this batch
-                                    return prev_yellows + current_yellows
+                                    # Convert to numeric types to handle both int and str
+                                    try:
+                                        # Handle empty strings
+                                        prev = float(prev_yellows) if prev_yellows != '' else 0.0
+                                        curr = float(current_yellows) if current_yellows != '' else 0.0
+                                        # Accumulate: previous + new cards from this batch
+                                        return int(prev + curr)
+                                    except (ValueError, TypeError):
+                                        # If conversion fails, return current value
+                                        return current_yellows
                             
                             return current_yellows
                         
@@ -462,24 +517,52 @@ class DatasetUpdater:
                 # Read existing data
                 df_existing = pd.read_csv(keepers_csv_path, encoding="utf-8")
                 
-                # Accumulate yellow cards for keepers
+                # Accumulate yellow cards for keepers PER COMPETITION_TYPE (league vs european)
                 if 'Players' in df_existing.columns and 'PlayersYellowCards' in df_existing.columns:
                     if 'Players' in df_keepers.columns and 'PlayersYellowCards' in df_keepers.columns:
-                        # Create dict with previous yellow cards per keeper
-                        previous_yellows = df_existing.groupby('Players')['PlayersYellowCards'].max().to_dict()
+                        # Create dict with previous yellow cards per keeper PER COMPETITION_TYPE
+                        # Key: (player_name, competition_type, team_name), Value: max yellow cards
+                        if 'competition_type' in df_existing.columns and 'team_name' in df_existing.columns:
+                            previous_yellows = df_existing.groupby(['Players', 'competition_type', 'team_name'])['PlayersYellowCards'].max().to_dict()
+                        else:
+                            # Fallback if columns don't exist
+                            previous_yellows = df_existing.groupby('Players')['PlayersYellowCards'].max().to_dict()
                         
                         # For each new keeper record, accumulate yellow cards
                         def accumulate_yellows(row):
                             player_name = row['Players']
                             current_yellows = row.get('PlayersYellowCards', 0)
+                            gameweek = row.get('gameweek', None)
                             
-                            # If keeper exists in previous data
-                            if player_name in previous_yellows:
-                                prev_yellows = previous_yellows[player_name]
+                            # CRITICAL: If gameweek is 1, start fresh (new season)
+                            if pd.notna(gameweek) and int(float(gameweek)) == 1:
+                                # New season starts, don't accumulate
+                                return current_yellows
+                            
+                            # Get competition_type and team_name if available
+                            if ('competition_type' in row.index and 'team_name' in row.index and 
+                                'competition_type' in df_existing.columns and 'team_name' in df_existing.columns):
+                                competition_type = row['competition_type']
+                                team_name = row['team_name']
+                                key = (player_name, competition_type, team_name)
+                            else:
+                                key = player_name
+                            
+                            # If keeper exists in previous data for this competition_type and team
+                            if key in previous_yellows:
+                                prev_yellows = previous_yellows[key]
                                 # If there's an increment, add it to the previous total
                                 if pd.notna(current_yellows) and pd.notna(prev_yellows):
-                                    # Accumulate: previous + new cards from this batch
-                                    return prev_yellows + current_yellows
+                                    # Convert to numeric types to handle both int and str
+                                    try:
+                                        # Handle empty strings
+                                        prev = float(prev_yellows) if prev_yellows != '' else 0.0
+                                        curr = float(current_yellows) if current_yellows != '' else 0.0
+                                        # Accumulate: previous + new cards from this batch
+                                        return int(prev + curr)
+                                    except (ValueError, TypeError):
+                                        # If conversion fails, return current value
+                                        return current_yellows
                             
                             return current_yellows
                         
@@ -520,6 +603,9 @@ class DatasetUpdater:
         """
         comp_lower = competition.lower()
 
+        if "uefa" in comp_lower:
+            return "european"
+
         if (
             "league" in comp_lower
             or "liga" in comp_lower
@@ -528,10 +614,8 @@ class DatasetUpdater:
             or "ligue" in comp_lower
         ):
             return "league"
-        elif "uefa" in comp_lower:
-            return "european"
-        else:
-            return "league"
+
+        return "league"
 
     def _prepare_target_matches(self, df: pd.DataFrame) -> pd.DataFrame:
         """
@@ -552,7 +636,7 @@ class DatasetUpdater:
         # Rename columns to match scraper expected format
         target_matches = target_matches.rename(
             columns={
-                "date_of_match": "date",
+                "date_of_match": "date_of_match",
                 "home_team_name": "home_team",
                 "away_team_name": "away_team",
             }
