@@ -19,10 +19,9 @@ class Settings:
         "players": "app/data/raw/players/players.csv",
         "keepers": "app/data/raw/players/keepers.csv",
         "ranking": "app/data/raw/rankings/{country}/{last_season[0]}/rankings_{competition}_{last_season[0]}.csv",
-        "dataset_global": "app/data/features/global/dataset.csv",
-        "dataset_country": "app/data/features/{country}/dataset_{country}.csv",
-        "dataset_competition": "app/data/features/{country}/{competition}/dataset_{competition}.csv",
+        "dataset": "app/data/features/dataset.csv",
         "teams": "app/data/raw/teams/teams.csv",
+        "stadiums": "app/data/raw/teams/stadiums.csv"
     }
 
     # Random state
@@ -683,6 +682,10 @@ class Settings:
                 "thresholds": [0, 10, 20, 30, 40, 50],
                 "operators": ["eq", "ge", "ge", "ge", "ge", "ge"],
             },
+            "unavailable_count": {
+                "thresholds": [0, 1, 2, 3, 4, 5, 6, 7],
+                "operators": ["eq", "eq", "eq", "eq", "eq", "eq", "eq", "ge"],
+            }
         },
     }
 
@@ -697,17 +700,26 @@ class Settings:
         # ==================== SPANISH COMPETITIONS ====================
         "liga": {
             "country": "spain",
-            "yellow_cards_suspensions": {
-                "type": "cycle",
-                "scope": "league_only",
-                "ban_matches": 1,
-                "cards_per_suspension": 5,
-                "cycle_resets_after_serving_ban": True,
+            "number_of_gameweeks": 38,
+            "champions_league": 4,
+            "europa_league": 3,
+            "conference_league": 0,
+            "relegated_teams": 3,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "cycle",
+                    "ban_matches": 1,
+                    "cards_per_suspension": 5,
+                    "cycle_resets_after_serving_ban": True,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/es/comps/12/{last_season[0]}/schedule/{last_season[0]}-La-Liga-Scores-and-Fixtures",
                 "ranking": "https://fbref.com/en/comps/12/{last_season[0]}/{last_season[0]}-La-Liga-Stats",
                 "fotmob": "https://www.fotmob.com/en/leagues/87/fixtures/laliga?season={last_season[0]}&group=by-date&page={page}",
+                "summer_transfers": "https://www.transfermarkt.co.uk/laliga/transfers/wettbewerb/ES1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+                "winter_transfers": "https://www.transfermarkt.co.uk/laliga/transfers/wettbewerb/ES1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
             },
             "betting_scraping_urls": {
                 "betclic": "https://www.betclic.fr/football-sfootball/espagne-laliga-c7",
@@ -723,19 +735,29 @@ class Settings:
         # ==================== ENGLISH COMPETITIONS ====================
         "premier_league": {
             "country": "england",
-            "yellow_cards_suspensions": {
-                "type": "cutoff_thresholds_by_team_match_number",
-                "scope": "premier_league_only",
-                "thresholds": [
-                    {"cards": 5, "up_to_team_match_inclusive": 19, "ban_matches": 1},
-                    {"cards": 10, "up_to_team_match_inclusive": 32, "ban_matches": 2},
-                    {"cards": 15, "up_to_team_match_inclusive": 38, "ban_matches": 3},
-                ],
+            "number_of_gameweeks": 38,
+            "champions_league": 4,
+            "europa_league": 3,
+            "conference_league": 0,
+            "relegated_teams": 3,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "milestones",
+                    "milestones": [
+                        {"cards": 5, "ban_matches": 1, "cutoff_league_fixtures": 19},
+                        {"cards": 10, "ban_matches": 2, "cutoff_league_fixtures": 32},
+                        {"cards": 15, "ban_matches": 3, "cutoff_league_fixtures": "end_of_season"},
+                    ],
+                    "cycle_resets_after_serving_ban": False,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/9/{last_season[0]}/schedule/{last_season[0]}-Premier-League-Scores-and-Fixtures",
                 "ranking": "https://fbref.com/es/comps/9/{last_season[0]}/{last_season[0]}-Premier-League-Stats",
                 "fotmob": "https://www.fotmob.com/en/leagues/47/fixtures/premier-league?group=by-date&season={last_season[0]}&group=by-date&page={page}",
+                "summer_transfers": "https://www.transfermarkt.co.uk/premier-league/transfers/wettbewerb/GB1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+                "winter_transfers": "https://www.transfermarkt.co.uk/premier-league/transfers/wettbewerb/GB1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
             },
             "betting_scraping_urls": {
                 "betclic": "https://www.betclic.fr/football-sfootball/angl-premier-league-c3",
@@ -751,16 +773,27 @@ class Settings:
         # ==================== ITALIAN COMPETITIONS ====================
         "serie_a": {
             "country": "italy",
-            "yellow_cards_suspensions": {
-                "type": "progressive_intervals",
-                "scope": "same_competition_only",
-                "ban_matches": 1,
-                "intervals_after_each_suspension": [5, 5, 5, 4, 3, 2, 1],  # then every further yellow
+            "number_of_gameweeks": 38,
+            "champions_league": 4,
+            "europa_league": 3,
+            "conference_league": 0,
+            "relegated_teams": 3,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "milestones",
+                        "ban_matches": 1,
+                        "milestones_cards": [5, 10, 14, 17, 19],
+                        "after_last_milestone": {"repeat_every_cards": 1, "starting_after_cards": 19},
+                        "cycle_resets_after_serving_ban": False,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/11/{last_season[0]}/schedule/{last_season[0]}-Serie-A-Scores-and-Fixtures",
                 "ranking": "https://fbref.com/es/comps/11/{last_season[0]}/{last_season[0]}-Serie-A-Stats",
                 "fotmob": "https://www.fotmob.com/leagues/55/fixtures/serie?group=by-date&season={last_season[0]}&group=by-date&page={page}",
+                "summer_transfers": "https://www.transfermarkt.co.uk/serie-a/transfers/wettbewerb/IT1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+                "winter_transfers": "https://www.transfermarkt.co.uk/serie-a/transfers/wettbewerb/IT1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
             },
             "betting_scraping_urls": {
                 "betclic": "https://www.betclic.fr/football-sfootball/angl-premier-league-c3", # Cambiar
@@ -776,17 +809,26 @@ class Settings:
         # ==================== FRENCH COMPETITIONS ====================
         "ligue_1": {
             "country": "france",
-            "yellow_cards_suspensions": {
-                "type": "rolling_window",
-                "scope": "all_official_matches",
-                "ban_matches": 1,
-                "window_matches": 10,
-                "cards_in_window_to_suspend": 3,
+            "number_of_gameweeks": 38,
+            "champions_league": 3,
+            "europa_league": 3,
+            "conference_league": 0,
+            "relegated_teams": 3,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "cycle",
+                    "ban_matches": 1,
+                    "cards_per_suspension": 5,
+                    "cycle_resets_after_serving_ban": True,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/13/{last_season[0]}/schedule/{last_season[0]}-Ligue-1-Scores-and-Fixtures",
                 "ranking": "https://fbref.com/es/comps/13/{last_season[0]}/{last_season[0]}-Ligue-1-Stats",
                 "fotmob": "https://www.fotmob.com/leagues/53/fixtures/ligue-1?group=by-date&season={last_season[0]}&group=by-date&page={page}",
+                "summer_transfers": "https://www.transfermarkt.co.uk/ligue-1/transfers/wettbewerb/FR1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+                "winter_transfers": "https://www.transfermarkt.co.uk/ligue-1/transfers/wettbewerb/FR1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
             },
             "betting_scraping_urls": {
                 "betclic": "https://www.betclic.fr/football-sfootball/angl-premier-league-c3", # Cambiar
@@ -802,17 +844,26 @@ class Settings:
         # ==================== GERMAN COMPETITIONS ====================
         "bundesliga": {
             "country": "germany",
-            "yellow_cards_suspensions": {
-                "type": "cycle",
-                "scope": "league_only",
-                "ban_matches": 1,
-                "cards_per_suspension": 5,
-                "cycle_resets_after_serving_ban": True,
+            "number_of_gameweeks": 34,
+            "champions_league": 4,
+            "europa_league": 3,
+            "conference_league": 0,
+            "relegated_teams": 3,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "cycle",
+                    "ban_matches": 1,
+                    "cards_per_suspension": 5,
+                    "cycle_resets_after_serving_ban": True,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/20/{last_season[0]}/schedule/{last_season[0]}-Bundesliga-Scores-and-Fixtures",
                 "ranking": "https://fbref.com/es/comps/20/{last_season[0]}/{last_season[0]}-Bundesliga-Stats",
                 "fotmob": "https://www.fotmob.com/leagues/54/fixtures/bundesliga?group=by-date&season={last_season[0]}&group=by-date&page={page}",
+                "summer_transfers": "https://www.transfermarkt.co.uk/bundesliga/transfers/wettbewerb/L1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+                "winter_transfers": "https://www.transfermarkt.co.uk/bundesliga/transfers/wettbewerb/L1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
             },
             "betting_scraping_urls": {
                 "betclic": "https://www.betclic.fr/football-sfootball/angl-premier-league-c3", # Cambiar
@@ -828,17 +879,27 @@ class Settings:
         # ==================== PORTUGESH COMPETITIONS ====================
         # "primeira_liga": {
         #     "country": "portugal",
-        #     "yellow_cards_suspensions": {
-        #         "type": "staggered_thresholds",
-        #         "scope": "same_competition_only",
-        #         "ban_matches": 1,
-        #         "thresholds": [5, 9, 12],
-        #         "after_last_threshold": {"every_n_cards": 3},
-        #     },
+        #     "number_of_gameweeks": 34,
+        #     "champions_league": 2,
+        #    "europa_league": 3,
+        #    "conference_league": 0,
+        #    "relegated_teams": 3,
+        #    "suspensions": {
+            #     "yellow_cards_suspensions": {
+            #         "type": "milestones",
+            #        "ban_matches": 1,
+            #        "milestones_cards": [5, 9, 12, 14],  # then every +2 cards after 14th
+            #        "after_last_milestone": {"repeat_every_cards": 2, "starting_after_cards": 14},
+            #        "cycle_resets_after_serving_ban": False,
+            #    },
+        #        "red_card_ban_matches": 1
+        #    },
         #     "information_scraping_urls": {
         #         "matches": "https://fbref.com/en/comps/32/{last_season[0]}/schedule/{last_season[0]}-Primeira-Liga-Scores-and-Fixtures",
         #         "ranking": "https://fbref.com/es/comps/33/{last_season[0]}/{last_season[0]}-Primeira-Liga-Stats",
         #         "fotmob": "https://www.fotmob.com/leagues/61/fixtures/liga-portugal?group=by-date&season={last_season[0]}&group=by-date&page={page}",
+        #        "summer_transfers": "https://www.transfermarkt.co.uk/liga-nos/transfers/wettbewerb/PO1/plus/?saison_id={last_season[0]}&s_w=s&leihe=1&intern=0",
+        #        "winter_transfers": "https://www.transfermarkt.co.uk/liga-nos/transfers/wettbewerb/PO1/plus/?saison_id={last_season[0]}&s_w=w&leihe=1&intern=0"
         #     },
         #     "betting_scraping_urls": {
         #         "betclic": "https://www.betclic.fr/football-sfootball/angl-premier-league-c3", # Cambiar
@@ -854,12 +915,14 @@ class Settings:
         # ==================== EUROPEAN COMPETITIONS ====================
         "uefa_champions_league": {
             "country": "europe",
-            "yellow_cards_suspensions": {
-                "type": "staggered_thresholds", # cambiar
-                "scope": "same_competition_only",
-                "ban_matches": 1,
-                "thresholds": [5, 9, 12],
-                "after_last_threshold": {"every_n_cards": 3},
+            "number_of_gameweeks": 98,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "milestones",
+                    "milestones_cards": [3, 5, 7, 9],
+                    "ban_matches": 1,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/8/{last_season[0]}/schedule/{last_season[0]}-Champions-League-Scores-and-Fixtures",
@@ -879,12 +942,14 @@ class Settings:
         },
         "uefa_europa_league": {
             "country": "europe",
-            "yellow_cards_suspensions": {
-                "type": "staggered_thresholds", # cambiar
-                "scope": "same_competition_only",
-                "ban_matches": 1,
-                "thresholds": [5, 9, 12],
-                "after_last_threshold": {"every_n_cards": 3},
+            "number_of_gameweeks": 98,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "milestones",
+                    "milestones_cards": [3, 5, 7, 9],
+                    "ban_matches": 1,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/19/{last_season[0]}/schedule/{last_season[0]}-Europa-League-Scores-and-Fixtures",
@@ -904,12 +969,14 @@ class Settings:
         },
         "uefa_conference_league": {
             "country": "europe",
-            "yellow_cards_suspensions": {
-                "type": "staggered_thresholds", # cambiar
-                "scope": "same_competition_only",
-                "ban_matches": 1,
-                "thresholds": [5, 9, 12],
-                "after_last_threshold": {"every_n_cards": 3},
+            "number_of_gameweeks": 98,
+            "suspensions": {
+                "yellow_cards_suspensions": {
+                    "type": "milestones",
+                    "milestones_cards": [3, 5, 7, 9],
+                    "ban_matches": 1,
+                },
+                "red_card_ban_matches": 1
             },
             "information_scraping_urls": {
                 "matches": "https://fbref.com/en/comps/882/{last_season[0]}/schedule/{last_season[0]}-Conference-League-Scores-and-Fixtures",
