@@ -100,5 +100,11 @@ def normalize_name(name: str) -> str:
     
     # 5) Remove leading/trailing underscores
     s = s.strip("_")
+    
+    # 6) Normalize common abbreviations
+    # Split by underscore, replace abbreviations, rejoin
+    words = s.split('_')
+    words = ['united' if word == 'utd' else word for word in words]
+    s = '_'.join(words)
 
     return s
\ No newline at end of file
